# Title: Thrifty Food Plan (TFP) Cost Estimates for Alaska and Hawaii Online Supplement R Code
# Project:		Thrifty Food Plan Cost Estimates for Alaska and Hawaii
# Author: U.S. Department of Agriculture. 
# Purpose:		Reproduce the results presented in the report using the data file. For additional
# information, please see the online supplement user guide.
# Inputs:		The data file: "TFPCostEstimatesforAlaskaandHawaii-OnlineSupplementDataFile.csv"
# Outputs:		"TFPCostEstimatesforAlaskaandHawaii-ReproducedResults.xlsx", 
# which contains output tables describing the following results for the final analysis:
#   - Overall price index value
#   - Market Basket Category price index values
#   - Number of unique products identified by their Universal Product Code (UPC) in the index
#   - Number of foods and beverages represented
#   - TFP costs and cost shares represented by foods and beverages
#_______________________________________________________________________________
# Table of contents
# 1) SETUP 
#     - install and load required packages
#     - set working directory
#     - read in data file
#     - set parameters
# 2) FUNCTIONS
#     - fun_index: function to calculate the overall price index
#     - fun_index_mbc: function to calculate the price index by Market Basket Category
#     - fun_coverage: function to calculate the coverage and sample size of the index
# 3) CALCULATION OF THE RESULTS
# 4) CREATE REPORT TABLES
# 5) WRITE RESULTS TO FILE
#_______________________________________________________________________________

# 1) SETUP --------------------------------------------------------------------

# Load packages and check for missing packages -> if package is missing, install
packages = c("tidyverse","openxlsx","readr")
package.check <- lapply(packages,
  FUN = function(x) {
    if (!require(x, character.only = TRUE)) {
      install.packages(x, dependencies = TRUE)
      library(x, character.only = TRUE)
    }
  }
)

# The user needs to set the appropriate working directory: 
wd <- "C:/Example-File-Path/"

# Read in online supplement data file: 
dt <- readr::read_csv(paste0(wd,"TFPCostEstimatesforAlaskaandHawaii-OnlineSupplementDataFile.csv"))

# Extract the labels of the analyses from the data file:
location_names <- base::unique(dt$Location)

# The TFP cost (monthly) for the mainland United States for June 2022 for the 
# reference family of four is available at:
# (https://fns-prod.azureedge.us/sites/default/files/media/file/CostofFoodJun2022Thrifty.pdf)
TFP_cost_mainland_June_2022 <- 939.90

# Sort Market Basket Categories in the order they appear in the report table
mbc_sort <- base::data.frame(Market_Basket_Category = c("Dark-green vegetables","Red and orange vegetables","Beans, peas, lentils","Starchy vegetables","Other vegetables",
                                                  "Whole fruit","100% fruit juice",
                                                  "Whole-grain staple grains","Whole-grain cereals","Refined-grain staple grains","Refined-grain other",
                                                  "Low- and non-fat milk, yogurt, soy alternatives","Higher fat milk, yogurt, soy alternatives","Cheese",
                                                  "Meats","Poultry","Eggs","Seafood","Nuts, seeds, soy products",
                                                  "Pre-prepared entrees and side dishes","Coffee and tea","Table fats and oils",
                                                  "Sauces, condiments, jams, honey, sugars, spices","Other foods and beverages"),
                   mbc_order = 1:24)

# Calculate TFP cost shares by Market Basket Category
TFP_cost_shares <- dt %>%
  dplyr::filter(EC_Form_Count == -9999 | EC_Form_Count == 1) %>%
  dplyr::group_by(Market_Basket_Category,Location) %>%
  dplyr::summarize(TFP_Cost_Share_Percent = base::round(base::sum(TFP_Cost_Share) *100,2)) %>%
  dplyr::select(-Location) %>%
  dplyr::distinct()

# 2) FUNCTIONS ----------------------------------------------------------------

# Function to replicate the results of the analyses (Table 3)
fun_index <- function(location_name){
  
  index <- dt %>%
    # Filter for results in the Location specified in the function
    dplyr::filter(Location == location_name) %>%
    # Drop observations for which unit price ratios are not available
    dplyr::filter(!Unit_Price_Ratio %in% c(-999,-9999)) %>%
    dplyr::summarize(value = base::sum(Unit_Price_Ratio * Index_Weight)) %>%
    # Add columns to data frame and re-order
    dplyr::mutate(Location = location_name,
                  variable = "Price index") %>%
    dplyr::select(Location,variable,value)
   
  return(index)
}

# Function to calculate the price index by TFP Market Basket Category (Table 4)
fun_index_mbc <- function(location_name){
  
  index_mbc <- dt %>%
    # Filter for results in the Location specified in the function
    dplyr::filter(Location == location_name) %>%
    # Drop observations for which unit price ratios are not available
    dplyr::filter(!Unit_Price_Ratio %in% c(-999,-9999)) %>%
    dplyr::group_by(Market_Basket_Category) %>%
    dplyr::mutate(Index_Weight_MBC = base::sum(Index_Weight)) %>%
    dplyr::summarize(value = base::sum((Index_Weight/Index_Weight_MBC) * Unit_Price_Ratio)) %>% 
    # Add columns to data frame and re-order
    dplyr::mutate(Location = location_name,
                variable = "Price index by Market Basket Category") %>%
    dplyr::select(Location,variable,Market_Basket_Category,value)

  return(index_mbc)
}

# Function to calculate the coverage of the analysis
fun_coverage <- function(location_name) {
  
  # Calculate the number of UPCs underlying the analysis
  upc_count <- dt %>%
    # Filter for results in the Location specified in the function
    dplyr::filter(Location == location_name) %>%
    # Drop observations with missing values
    dplyr::filter(!Common_Set_UPC_Count %in% c(-999,-9999)) %>%
    # Add up UPCs in the common set
    dplyr::summarize(value = base::sum(Common_Set_UPC_Count, na.rm = TRUE)) %>%
    # Add columns to data frame and re-order
    dplyr::mutate(Location = location_name,
                  variable = "Number of UPCs in index") %>%
    dplyr::select(Location,variable,value)
   
  # Calculate number of EC-Form combinations ('foods and beverages') represented 
  ecform_count <- dt %>%
    # Filter for results in the Location specified in the function
    dplyr::filter(Location == location_name) %>%
    # Drop observations for which unit prices are not available 
    dplyr::filter(!Unit_Price_Ratio %in% c(-999,-9999)) %>%
    # Add up EC-Forms in the common set
    dplyr::summarize(value = base::sum(EC_Form_Count)) %>%
    # Add columns to data frame and re-order
    dplyr::mutate(Location = location_name,
                  variable = "Number of foods and beverages represented") %>%
    dplyr::select(Location,variable,value)
  
  # Calculate the TFP cost ($) represented by foods and beverages
  tfp_cost <- dt %>%
    # Filter for results in the Location specified in the function
    dplyr::filter(Location == location_name) %>%
    # Drop observations for which unit prices are not available 
    dplyr::filter(!Unit_Price_Ratio %in% c(-999,-9999)) %>%
    # Add up TFP cost ($) in the common set
    dplyr::summarize(value = base::round(base::sum(Weekly_TFP_Cost),1)) %>% 
    # Add columns to data frame and re-order
    dplyr::mutate(Location = location_name,
                  variable = "TFP cost represented by foods and beverages") %>%
    dplyr::select(Location,variable,value)
  
  # Calculate the percentage of TFP cost ($) represented by foods and beverages
  percent_tfp_cost <- dt %>%
    # Filter for results in the Location specified in the function
    dplyr::filter(Location == location_name) %>%
    # Drop observations for which unit prices are not available 
    dplyr::filter(!Unit_Price_Ratio %in% c(-999,-9999)) %>%
    # Add up TFP cost ($) in the common set
    dplyr::summarize(value = base::round(base::sum(TFP_Cost_Share * 100), 1)) %>% 
    # Add columns to data frame and re-order
    dplyr::mutate(Location = location_name,
                  variable = "Percentage of TFP cost represented by foods and beverages") %>%
    dplyr::select(Location,variable,value)
  
  # Put together data frames into one table:
  coverage <- base::rbind(upc_count,ecform_count,tfp_cost,percent_tfp_cost) %>%
    dplyr::rename(Measure = variable)
  
  return(coverage)
  
}

# 3) CALCULATION OF THE RESULTS ----------------------------------------------------------------

# Using the function "fun_index" to calculate the price index for Anchorage and Honolulu:
index_main_analysis_honolulu <- fun_index("Honolulu")
index_main_analysis_anchorage <- fun_index("Anchorage")

# Using the function "fun_index_mbc" to calculate the price index for Anchorage and Honolulu 
# at the Market Basket Category level:
index_mbc_main_analysis_honolulu <- fun_index_mbc("Honolulu")
index_mbc_main_analysis_anchorage <- fun_index_mbc("Anchorage")

# Using the function "fun_coverage" to calculate the information about the sample size and coverage
# for the analyses
coverage_anchorage <- fun_coverage("Anchorage")
coverage_honolulu <- fun_coverage("Honolulu")



# 4) CREATE REPORT TABLES ------------------------------------------------------------ 

# Table 3
table_3 <- rbind(fun_index("Anchorage"),fun_index("Honolulu")) %>%
  tidyr::pivot_wider(names_from ="variable",values_from="value") %>%
  dplyr::rename(price_index = 'Price index') %>%
  dplyr::mutate(tfp_cost = base::round(TFP_cost_mainland_June_2022 * price_index, 1),
                price_index = base::round(price_index, 4)) 

# Table 4:
table_4 <- base::rbind(fun_index_mbc("Anchorage"),fun_index_mbc("Honolulu")) %>%
  tidyr::pivot_wider(names_from = "Location", values_from="value") %>%
  dplyr::select(-variable) %>%
  dplyr::full_join(mbc_sort, by="Market_Basket_Category") %>%
  dplyr::arrange(mbc_order) %>%
  dplyr::select(-mbc_order) %>%
  dplyr::full_join(TFP_cost_shares, by="Market_Basket_Category") %>%
  dplyr::select(Market_Basket_Category, TFP_Cost_Share_Percent, Anchorage, Honolulu) %>%
  dplyr::mutate(Anchorage = base::round(Anchorage,3),
                Honolulu = base::round(Honolulu,3))

# Calculate results and sample size for in Anchorage and Honolulu
other_summary_stats <- list()
for (i in location_names){
  other_summary_stats <- base::rbind(other_summary_stats,fun_coverage(i))
  }

# 5) WRITE RESULTS TO FILE -----------------------------------------------------
# Create list of results:
excel_file <- list("Table 3" = table_3,
                   "Table 4" = table_4,
                   "Other Summary Statistics" = other_summary_stats)

# Write list to file:
openxlsx::write.xlsx(excel_file, 
                     paste0(wd, "TFPCostEstimatesforAlaskaandHawaii-ReproducedResults.xlsx"),
                     overwrite = TRUE)
# END OF CODE