/**********************************************************************************************************
 Program Name:	Thrifty Food Plan (TFP) Cost Estimates for Alaska and Hawaii Online Supplement SAS Code
 Project:		Thrifty Food Plan Cost Estimates for Alaska and Hawaii
 Programmer:	U.S. Department of Agriculture
 Purpose:		Reproduce the results presented in the report using the data file. For additional
				information, please see the online supplement user guide.
 Inputs:		The data file: TFPCostEstimatesforAlaskaandHawaii-OnlineSupplementDataFile.csv
 Outputs:		TFPCostEstimatesforAlaskaandHawaii-ReproducedResults.xlsx, which contains output tables 
				describing the following results for the final analysis:
					- Overall price index value
					- Market Basket Category price index values
					- Number of unique products identified by their Universal Product Code (UPC) in the index
					- Number of foods and beverages represented
					- TFP costs and cost shares represented by foods and beverages
**********************************************************************************************************/
/*Step 1: Define the file pathway that the online supplement has been saved to and assign file references to input and output files*/
	%let path = C:\example-file-path;
	filename file_in "&path.\TFPCostEstimatesforAlaskaandHawaii-OnlineSupplementDataFile.csv";
	filename file_out "&path.\TFPCostEstimatesforAlaskaandHawaii-ReproducedResults.xlsx";

/*Step 2: Read in the data file, recode missing values for convenience, and sort*/
	proc import file = file_in
				dbms = csv
				out = file_in
				replace;
				guessingrows = max;
	run;

	data puf;
		set file_in;
		if unit_price_ratio <0 then unit_price_ratio = .;
		if common_set_upc_count <0 then common_set_upc_count = .;
		if ec_form_count <0 then ec_form_count = .;
	run;

	proc sort data=puf;
		by location ec form;
	run;

/*Step 3: Reproduce results*/
	/*Calculate overall index value and TFP cost estimates*/
		*the overall index value is the arithmetic mean unit price ratio weighted by the index weight;
		proc summary data=puf nway;
			class location;
			weight index_weight;
			var unit_price_ratio;
			output out=overall_means(drop=_:) mean=;
		run;

		*Multiply the index value by June 2022 cost of the TFP in the mainland United States and round to the nearest $0.10 to yield monthly cost in Alaska and Hawaii;
		data table3;
			set overall_means;
			unit_price_ratio = round(unit_price_ratio,0.0001);
			tfp_cost = round(unit_price_ratio * 939.90,0.1);
			rename unit_price_ratio = price_index;
		run;

	/*Market Basket Category index values for main analysis*/
		*Calculate mean unit price ratio weighted by the index weight for each Market Basket Category;
		proc summary data=puf nway;
			class market_basket_category location;
			weight index_weight;
			var unit_price_ratio;
			output out=mbc_means(drop=_:) mean=unit_price_ratio sumwgt=TFP_Cost_Share_Percent;
		run;

		*Assign row number to each Market Basket Category and round values to match Table 4 of the report;
		data mbc_means_ordered;
			set mbc_means;
			TFP_Cost_Share_Percent = round(TFP_Cost_Share_Percent,0.0001) * 100;
			unit_price_ratio = round(unit_price_ratio,0.001); 
			if market_basket_category = "Dark-green vegetables" then row_number = 1;
			else if market_basket_category = "Red and orange vegetables" then row_number = 2;
			else if market_basket_category = "Beans, peas, lentils" then row_number = 3;
			else if market_basket_category = "Starchy vegetables" then row_number = 4;
			else if market_basket_category = "Other vegetables" then row_number = 5;
			else if market_basket_category = "Whole fruit" then row_number = 6;
			else if market_basket_category = "100% fruit juice" then row_number = 7;
			else if market_basket_category = "Whole-grain staple grains" then row_number = 8;
			else if market_basket_category = "Whole-grain cereals" then row_number = 9;
			else if market_basket_category = "Refined-grain staple grains" then row_number = 10;
			else if market_basket_category = "Refined-grain other" then row_number = 11;
			else if market_basket_category = "Low- and non-fat milk, yogurt, soy alternatives" then row_number = 12;
			else if market_basket_category = "Higher fat milk, yogurt, soy alternatives" then row_number = 13;
			else if market_basket_category = "Cheese" then row_number = 14;
			else if market_basket_category = "Meats" then row_number = 15;
			else if market_basket_category = "Poultry" then row_number = 16;
			else if market_basket_category = "Eggs" then row_number = 17;
			else if market_basket_category = "Seafood" then row_number = 18;
			else if market_basket_category = "Nuts, seeds, soy products" then row_number = 19;
			else if market_basket_category = "Pre-prepared entrees and side dishes" then row_number = 20;
			else if market_basket_category = "Coffee and tea" then row_number = 21;
			else if market_basket_category = "Table fats and oils" then row_number = 22;
			else if market_basket_category = "Sauces, condiments, jams, honey, sugars, spices" then row_number = 23;
			else if market_basket_category = "Other foods and beverages" then row_number = 24;
		run;

		*Sort by row number assigned in previous step;
		proc sort data=mbc_means_ordered;
			by row_number market_basket_category TFP_Cost_Share_Percent;
		run;
		
		*Transpose for export, drop row number;
		proc transpose data=mbc_means_ordered out=table4(drop=_: row_number);
			by row_number market_basket_category TFP_Cost_Share_Percent;
			id location;
			var unit_price_ratio;
		run;

	/*Other summary statistics*/
		*Calculate number of UPCs, foods and beverages, and TFP cost coverage;
		proc summary data=puf nway;
			where index_weight > 0;
			by location;
			var common_set_upc_count ec_form_count weekly_tfp_cost tfp_cost_share;
			output out= summary_stats(drop=_:) sum=;
		run;

		*Label, round, and transpose for export, multiply percentage by 100;
		data summary_stats1;
			set summary_stats;
			by location;
			label common_set_upc_count = "Number of UPCs in Index"
					ec_form_count = "Number of foods and beverages represented"
					weekly_tfp_cost = "TFP cost represented by foods and beverages"
					tfp_cost_share = "Percentage of TFP cost represented by foods and beverages";
			tfp_cost_share = round(tfp_cost_share * 100,0.1);
			weekly_tfp_cost = round(weekly_tfp_cost, 0.1);
		run;

		proc transpose data=summary_stats1 out=summary_stats2(drop=_name_ rename=(_label_ = Measure col1 = Value));
			by location;
			var common_set_upc_count ec_form_count weekly_tfp_cost tfp_cost_share;
		run;

/*Step 4: Export results*/
	proc export data = table3
				file = file_out
				dbms = xlsx
				replace;
				sheet = "Table 3";
	run;
	proc export data = table4
				file = file_out
				dbms = xlsx
				replace;
				sheet = "Table 4";
	run;
	proc export data = summary_stats2
				file = file_out
				dbms = xlsx
				replace;
				sheet = "Other Summary Statistics";
	run;
